import { Config } from '../config/argv';
import { Entry } from './Entry';
export declare type SetConfig = (config: Config) => Config | Promise<Config>;
export declare type OnCreateBabelConfig = (params: any, dev: boolean) => void;
export declare type OnCreateWebpackConfig<C = any> = (config: C, dev: boolean, args: Config) => C;
export declare type ModifyFiles = (files: string[], args: Config) => string[];
export declare type ModifyEntry = (entry: Entry, args: Config) => Entry;
export declare type OnCreateDevServer = <A>(app: A) => void;
export declare type OnPreBuild = (args: Config) => void;
export declare type OnPostBuild = (args: Config) => void;
export declare type OnPreRender = () => void;
export declare type OnPostRender = () => void;
export interface PluginFactory {
    setConfig?: SetConfig;
    onCreateBabelConfig?: OnCreateBabelConfig;
    onCreateDevServer?: OnCreateDevServer;
    onCreateWebpackConfig?: OnCreateWebpackConfig;
    modifyFiles?: ModifyFiles;
    modifyEntry?: ModifyEntry;
    onPreBuild?: OnPreBuild;
    onPostBuild?: OnPostBuild;
}
export declare class Plugin<C = any> implements PluginFactory {
    static runPluginsMethod(plugins: Plugin[] | undefined): (method: keyof Plugin, ...args: any[]) => void;
    static propsOfPlugins(plugins: Plugin[]): (prop: keyof Plugin) => any[];
    static reduceFromPlugins<C>(plugins: Plugin[] | undefined): (method: keyof Plugin, initial: C, ...args: any[]) => C;
    static reduceFromPluginsAsync<C>(plugins: Plugin[] | undefined): (method: keyof Plugin, initial: C, ...args: any[]) => Promise<C>;
    readonly setConfig?: SetConfig;
    readonly onCreateWebpackConfig?: OnCreateWebpackConfig<C>;
    readonly onCreateBabelConfig?: OnCreateBabelConfig;
    readonly modifyFiles?: ModifyFiles;
    readonly modifyEntry?: ModifyEntry;
    readonly onCreateDevServer?: OnCreateDevServer;
    readonly onPreBuild?: OnPreBuild;
    readonly onPostBuild?: OnPostBuild;
    constructor(p: PluginFactory);
}
export declare function createPlugin<C = any>(factory: PluginFactory): Plugin<C>;
